@@ -137,9 +137,13 @@ class TestIntegrationCluster < TestIntegration
 
   def test_after_booted_and_after_stopped
     skip_unless_signal_exist? :TERM
-    cli_server "-w #{workers} -C test/config/event_after_booted_and_after_stopped.rb -C test/config/event_after_booted_exit.rb test/rackup/hello.ru"
+    cli_server "-w #{workers} " \
+      "-C test/config/event_after_booted_and_after_stopped.rb " \
+      "-C test/config/event_after_booted_exit.rb " \
+      "test/rackup/hello.ru",
+      no_wait: true
 
-    # above checks 'Ctrl-C', below is logged after workers boot
+    # below is logged after workers boot
     assert wait_for_server_to_include('after_booted called')
     assert wait_for_server_to_include('Goodbye!')
     # below logged after workers are stopped
@@ -480,10 +484,40 @@ class TestIntegrationCluster < TestIntegration
     cli_server "-w #{workers} --preload test/rackup/write_to_stdout_on_boot.ru"
 
     get_worker_pids
-    loading_app_count = @server_log.scan('Loading app').length
+    loading_app_count = @server_log.scan("Loading app").length
     assert_equal 1, loading_app_count
   end
 
+  def test_application_is_loaded_exactly_once_if_using_fork_worker
+    cli_server "-w #{workers} --fork-worker test/rackup/write_to_stdout_on_boot.ru"
+
+    get_worker_pids
+    loading_app_count = @server_log.scan("Loading app").length
+    assert_equal 1, loading_app_count # loaded in worker 0
+
+    @server_log.clear
+    Process.kill :SIGURG, @pid
+
+    get_worker_pids 1, workers - 1
+    loading_app_count = @server_log.scan("Loading app").length
+    assert_equal 0, loading_app_count
+  end
+
+  def test_application_is_loaded_exactly_once_if_using_preload_app_with_fork_worker
+    cli_server "-w #{workers} --preload --fork-worker test/rackup/write_to_stdout_on_boot.ru"
+
+    get_worker_pids
+    loading_app_count = @server_log.scan("Loading app").length
+    assert_equal 1, loading_app_count # loaded in master
+
+    @server_log.clear
+    Process.kill :SIGURG, @pid
+
+    get_worker_pids 1, workers - 1
+    loading_app_count = @server_log.scan("Loading app").length
+    assert_equal 0, loading_app_count
+  end
+
   def test_warning_message_outputted_when_single_worker
     cli_server "-w 1 test/rackup/hello.ru"
 
