@@ -1630,7 +1630,7 @@ class TestPumaServer < PumaTest
 
     response = send_http_read_response "HEAD / HTTP/1.0\r\n\r\n"
 
-    refute_match(/[\r\n]Cookie: hack[\r\n]/, response)
+    refute_includes response.headers, 'Cookie: hack'
   end
 
   # HTTP Injection Tests
@@ -1952,7 +1952,7 @@ class TestPumaServer < PumaTest
     assert_equal @host, body
   end
 
-  # see https://github.com/sinatra/sinatra/blob/master/examples/stream.ru
+  # see https://github.com/sinatra/sinatra/blob/main/examples/stream.ru
   def test_streaming_enum_body_1
     str = "Hello Puma World"
     body_len = str.bytesize * 3
